﻿import com.Components.AdvancedButton;
import com.GameInterface.Chat;
import com.GameInterface.Game.Character;
import com.GameInterface.Game.CharacterBase;
import com.GameInterface.Log;
import com.Utils.Signal;
import Enums.AttackResultType;
import Enums.Dummy;
import flash.geom.Point;
import mx.utils.Delegate;
import com.Utils.LDBFormat;
import com.Utils.ID32;
import com.GameInterface.Tooltip.TooltipUtils;

class DPSWindow extends MovieClip {
	public static var MODE_DAMAGE: Number = 0;
	public static var MODE_HEAL: Number = 1;
	public static var MODES_COUNT: Number = 2;
	
	static public var Instance:DPSWindow;
	
	public var SignalPositionChanged:Signal;
	public var SignalClosedWindow:Signal;
	
	private var m_title: TextField;
	private var m_total: InfoLine;
	private var m_time: InfoLine;
	private var m_perSec: InfoLine;
	private var m_avrg: InfoLine;
	
	private var m_detail:DetailPane;
	
	private var m_Background:MovieClip;
	private var m_BackgroundFull:MovieClip;
	private var m_backgroundHeight:Number;
	
	private var m_CloseButton:AdvancedButton;

	private var m_ChatButton:AdvancedButton;

	private var m_PrevButton:AdvancedButton;
	private var m_NextButton:AdvancedButton;
	private var m_FullButton:AdvancedButton;
	
	private var m_mode: Number;
	
	
	private var m_Dragging:Boolean = false;

	private var m_stat:Statistics;
	var m_ClientCharacter:Character;
	
	public function DPSWindow() {
		super();
		Instance = this;
		com.Utils.GlobalSignal.SignalDamageNumberInfo.Connect( SlotDamageInfo, this );

		SignalClosedWindow = new Signal();
		SignalPositionChanged = new Signal();
		m_Background.onRelease = Delegate.create(this, handleStopDrag);
		m_Background.onReleaseOutside = Delegate.create(this, handleStopDrag);
		m_Background.onPress = Delegate.create(this, handleStartDrag);
		
		onEnterFrame = Delegate.create(this, UpdateStatistics);
		
		m_stat = new Statistics();

		m_mode = MODE_DAMAGE;
		m_title.text = getModeName();
	}
	
	public function onLoad() {
		m_NextButton.SignalButtonSelected.Connect(NextMode, this);
		m_PrevButton.SignalButtonSelected.Connect(PrevMode, this);
		
		m_FullButton.SignalButtonSelected.Connect(ToggleView, this);
		
		m_CloseButton.SignalButtonSelected.Connect(CloseWindow, this);
		
		m_ChatButton.SignalButtonSelected.Connect(SendToChat, this);
		
		m_BackgroundFull._visible = false;
		m_backgroundHeight = m_Background._height;
		
		// test all modes
		ToShortView();
		ToFullView();
		
		ToFullView();
		
		m_ClientCharacter = Character.GetClientCharacter();
	}
	
	private function SendToChat():Void 
	{
		if(m_mode == MODE_DAMAGE)
			Chat.SetChatInput(" " + m_ClientCharacter.GetName() + " - damage: " + m_stat.damageDealt + " dps: " + m_stat.damagePerSec + " time: " + m_stat.damageTime + "\n ");
		else if (m_mode == MODE_HEAL)
		{
			Chat.SetChatInput(" " + m_ClientCharacter.GetName() + " - heal: " + m_stat.healDealt + " hps: " + m_stat.healPerSec + " time: " + m_stat.healTime + "\n ");
			//Chat.SetChatInput("");
		}
	}
	
	private function ToggleView():Void
	{
		if (m_detail._visible)
			ToShortView();
		else
			ToFullView();
	}
	
	private function ToFullView():Void 
	{
		m_detail._visible = true;
		m_Background._height = m_BackgroundFull._height;
	}
	
	private function ToShortView():Void 
	{
		m_detail._visible = false;
		m_Background._height = m_backgroundHeight;
	}
	
	private function NextMode():Void 
	{
		m_mode++;
		if (m_mode >= MODES_COUNT)
			m_mode = MODE_DAMAGE;
		
		m_FullButton._visible = (m_mode != MODE_HEAL);
		if (m_mode != MODE_DAMAGE)
			ToShortView();
			
		m_title.text = getModeName();
	}

	private function PrevMode():Void 
	{
		m_mode--;
		if (m_mode < MODE_DAMAGE)
			m_mode = MODE_HEAL;
			
		m_FullButton._visible = (m_mode != MODE_HEAL);
		if (m_mode != MODE_DAMAGE)
			ToShortView();
		
		m_title.text = getModeName();
	}
	
	private function getModeName():String
	{
		switch(m_mode)
		{
			case MODE_DAMAGE:
				return "Damage";
			case MODE_HEAL:
				return "Heal";
		}
		return "???";
	}
	
	public function CloseWindow(eventObj:Object):Void {
		SignalClosedWindow.Emit(this);
	}
	
	public function UpdateStatistics():Void 
	{
		m_stat.update();
		
		if (m_detail._visible)
			m_detail.update(m_stat);
		
		var total:Number = 0;
		var time:Number = 0;
		var perSec:Number = 0;
		var avrg:Number = 0;
		
		switch(m_mode)
		{
			case MODE_DAMAGE:
			{
				total = m_stat.damageDealt;
				time = m_stat.damageTime;
				perSec = m_stat.damagePerSec;
				avrg = m_stat.damageAvrg;

				break;
			}
			case MODE_HEAL:
			{
				total = m_stat.healDealt;
				time = m_stat.healTime;
				perSec = m_stat.healPerSec;
				avrg = m_stat.healAvrg;
//				Log.Error("Stats","Healing: " + m_stat.healTime + " " + m_stat.healDealt + " " + m_stat.healPerSec + " " + m_stat.healAvrg);
				break;
			}
		}
		
		m_total.label = "Total: ";
		m_time.label = "Battle time: ";
		m_perSec.label = "Per second: ";
		m_avrg.label = "Avarage:";

		m_total.value = total;
		m_time.value = time;
		m_perSec.value = perSec;
		m_avrg.value = avrg;
	}
	
	private function handleStartDrag() {
		this.startDrag();
	}
	
	private function handleStopDrag(buttonIdx:Number) {
		this.stopDrag();
		SignalPositionChanged.Emit(this._x, this._y);
	}
	
	private function RemoveSelection() {
		Selection.setFocus(null);
	}
	
	function SlotDamageInfo( statID:Number, damage:Number, absorb:Number, 
			attackResultType:Number, 
			attackType:Number,  
			attackOffensiveLevel:Number, 
			attackDefensiveLevel:Number, 
			context:Number, 
			targetID:ID32, 
			iconID:ID32, 
			iconColorLine:Number, 
			combatLogFeedbackType:Number )
	{
		var rec : StatRecord = new StatRecord(statID, damage, absorb, attackResultType, attackType, attackOffensiveLevel, attackDefensiveLevel, context, targetID, iconID, iconColorLine, combatLogFeedbackType);
		m_stat.Add(rec);
	}

}
